@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,11 +35,14 @@
 #include <glib/gi18n.h>
 #include <string.h>
 
-#include <common/na-object-api.h>
+#include <api/na-object-api.h>
+
+#include <core/na-io-provider.h>
 
 #include "base-window.h"
 #include "nact-application.h"
 #include "nact-main-statusbar.h"
+#include "nact-gtk-utils.h"
 #include "nact-iactions-list.h"
 #include "nact-main-tab.h"
 #include "nact-iaction-tab.h"
@@ -64,6 +67,7 @@ enum {
 
 static gboolean st_initialized = FALSE;
 static gboolean st_finalized = FALSE;
+static gboolean st_on_selection_change = FALSE;
 
 static GType         register_type( void );
 static void          interface_base_init( NactIActionTabInterface *klass );
@@ -71,9 +75,10 @@ static void          interface_base_finalize( NactIActionTabInterface *klass );
 
 static void          on_iactions_list_column_edited( NactIActionTab *instance, NAObject *object, gchar *text, gint column );
 static void          on_tab_updatable_selection_changed( NactIActionTab *instance, gint count_selected );
+static void          on_tab_updatable_provider_changed( NactIActionTab *instance, NAObjectItem *item );
 
 static void          on_target_selection_toggled( GtkToggleButton *button, NactIActionTab *instance );
-static void          on_target_background_toggled( GtkToggleButton *button, NactIActionTab *instance );
+static void          on_target_location_toggled( GtkToggleButton *button, NactIActionTab *instance );
 
 static void          check_for_label( NactIActionTab *instance, GtkEntry *entry, const gchar *label );
 static void          on_label_changed( GtkEntry *entry, NactIActionTab *instance );
@@ -84,22 +89,27 @@ static void          on_target_toolbar_toggled( GtkToggleButton *button, NactIAc
 static void          on_toolbar_same_label_toggled( GtkToggleButton *button, NactIActionTab *instance );
 static void          toolbar_same_label_set_sensitive( NactIActionTab *instance, NAObjectItem *item );
 
+static void          setup_toolbar_label( NactIActionTab *instance, NAObjectItem *item, const gchar *label );
 static void          on_toolbar_label_changed( GtkEntry *entry, NactIActionTab *instance );
 static void          toolbar_label_set_sensitive( NactIActionTab *instance, NAObjectItem *item );
 
 static void          on_tooltip_changed( GtkEntry *entry, NactIActionTab *instance );
 
 static GtkTreeModel *create_stock_icon_model( void );
-static void          display_icon( NactIActionTab *instance, GtkWidget *image, gboolean display );
 static void          icon_combo_list_fill( GtkComboBoxEntry* combo );
 static void          on_icon_browse( GtkButton *button, NactIActionTab *instance );
 static void          on_icon_changed( GtkEntry *entry, NactIActionTab *instance );
 static gint          sort_stock_ids( gconstpointer a, gconstpointer b );
 static gchar        *strip_underscore( const gchar *text );
+static void          release_icon_combobox( NactIActionTab *instance );
 
 static GtkButton    *get_enabled_button( NactIActionTab *instance );
 static void          on_enabled_toggled( GtkToggleButton *button, NactIActionTab *instance );
 
+static void          on_readonly_toggled( GtkToggleButton *button, NactIActionTab *instance );
+
+static void          display_provider_name( NactIActionTab *instance, NAObjectItem *item );
+
 GType
 nact_iaction_tab_get_type( void )
 {
@@ -145,6 +155,7 @@ interface_base_init( NactIActionTabInterface *klass )
 	static const gchar *thisfn = "nact_iaction_tab_interface_base_init";
 
 	if( !st_initialized ){
+
 		g_debug( "%s: klass=%p", thisfn, ( void * ) klass );
 
 		klass->private = g_new0( NactIActionTabInterfacePrivate, 1 );
@@ -158,28 +169,54 @@ interface_base_finalize( NactIActionTabInterface *klass )
 {
 	static const gchar *thisfn = "nact_iaction_tab_interface_base_finalize";
 
-	if( !st_finalized ){
+	if( st_initialized && !st_finalized ){
+
 		g_debug( "%s: klass=%p", thisfn, ( void * ) klass );
 
-		g_free( klass->private );
-
 		st_finalized = TRUE;
+
+		g_free( klass->private );
 	}
 }
 
+/*
+ * GTK_ICON_SIZE_MENU         : 16x16
+ * GTK_ICON_SIZE_SMALL_TOOLBAR: 18x18
+ * GTK_ICON_SIZE_LARGE_TOOLBAR: 24x24
+ * GTK_ICON_SIZE_BUTTON       : 20x20
+ * GTK_ICON_SIZE_DND          : 32x32
+ * GTK_ICON_SIZE_DIALOG       : 48x48
+ *
+ * icon is rendered for GTK_ICON_SIZE_MENU (na_object_item_get_pixbuf)
+ */
 void
 nact_iaction_tab_initial_load_toplevel( NactIActionTab *instance )
 {
 	static const gchar *thisfn = "nact_iaction_tab_initial_load_toplevel";
 	GtkWidget *icon_widget;
+	GtkTreeModel *model;
+	GtkButton *button;
+	GtkRequisition requisition;
+	GtkFrame *frame;
+	gint size;
 
 	g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
 	g_return_if_fail( NACT_IS_IACTION_TAB( instance ));
 
 	if( st_initialized && !st_finalized ){
 
+		button = GTK_BUTTON( base_window_get_widget( BASE_WINDOW( instance ), "ActionIconBrowseButton" ));
+		gtk_widget_size_request( GTK_WIDGET( button ), &requisition );
+		g_debug( "%s: button requisition width=%d, height=%d", thisfn, requisition.width, requisition.height );
+		frame = GTK_FRAME( base_window_get_widget( BASE_WINDOW( instance ), "ActionIconFrame" ));
+		size = requisition.height - 4;
+		gtk_widget_set_size_request( GTK_WIDGET( frame ), size, size );
+		gtk_frame_set_shadow_type( frame, GTK_SHADOW_IN );
+
 		icon_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionIconComboBoxEntry" );
-		gtk_combo_box_set_model( GTK_COMBO_BOX( icon_widget ), create_stock_icon_model());
+		model = create_stock_icon_model();
+		gtk_combo_box_set_model( GTK_COMBO_BOX( icon_widget ), model );
+		g_object_unref( model );
 		icon_combo_list_fill( GTK_COMBO_BOX_ENTRY( icon_widget ));
 	}
 }
@@ -189,95 +226,108 @@ nact_iaction_tab_runtime_init_toplevel( NactIActionTab *instance )
 {
 	static const gchar *thisfn = "nact_iaction_tab_runtime_init_toplevel";
 	GtkWidget *label_widget, *tooltip_widget, *icon_widget;
-	GtkWidget *button;
 	GtkButton *enabled_button;
+	GtkWidget *button;
 
 	g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
 	g_return_if_fail( NACT_IS_IACTION_TAB( instance ));
 
 	if( st_initialized && !st_finalized ){
 
-		g_signal_connect(
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( instance ),
-				TAB_UPDATABLE_SIGNAL_SELECTION_CHANGED,
-				G_CALLBACK( on_tab_updatable_selection_changed ),
-				instance );
+				MAIN_WINDOW_SIGNAL_SELECTION_CHANGED,
+				G_CALLBACK( on_tab_updatable_selection_changed ));
 
-		g_signal_connect(
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
+				G_OBJECT( instance ),
+				TAB_UPDATABLE_SIGNAL_PROVIDER_CHANGED,
+				G_CALLBACK( on_tab_updatable_provider_changed ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( instance ),
 				IACTIONS_LIST_SIGNAL_COLUMN_EDITED,
-				G_CALLBACK( on_iactions_list_column_edited ),
-				instance );
+				G_CALLBACK( on_iactions_list_column_edited ));
 
 		button = base_window_get_widget( BASE_WINDOW( instance ), "ActionTargetSelectionButton" );
-		g_signal_connect(
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( button ),
 				"toggled",
-				G_CALLBACK( on_target_selection_toggled ),
-				instance );
+				G_CALLBACK( on_target_selection_toggled ));
 
-		button = base_window_get_widget( BASE_WINDOW( instance ), "ActionTargetBackgroundButton" );
-		g_signal_connect(
+		button = base_window_get_widget( BASE_WINDOW( instance ), "ActionTargetLocationButton" );
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( button ),
 				"toggled",
-				G_CALLBACK( on_target_background_toggled ),
-				instance );
+				G_CALLBACK( on_target_location_toggled ));
 
 		label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionMenuLabelEntry" );
-		g_signal_connect(
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( label_widget ),
 				"changed",
-				G_CALLBACK( on_label_changed ),
-				instance );
+				G_CALLBACK( on_label_changed ));
 
 		button = base_window_get_widget( BASE_WINDOW( instance ), "ActionTargetToolbarButton" );
-		g_signal_connect(
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( button ),
 				"toggled",
-				G_CALLBACK( on_target_toolbar_toggled ),
-				instance );
+				G_CALLBACK( on_target_toolbar_toggled ));
 
 		button = base_window_get_widget( BASE_WINDOW( instance ), "ToolbarSameLabelButton" );
-		g_signal_connect(
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( button ),
 				"toggled",
-				G_CALLBACK( on_toolbar_same_label_toggled ),
-				instance );
+				G_CALLBACK( on_toolbar_same_label_toggled ));
 
-		label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionIconLabelEntry" );
-		g_signal_connect(
+		label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionToolbarLabelEntry" );
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( label_widget ),
 				"changed",
-				G_CALLBACK( on_toolbar_label_changed ),
-				instance );
+				G_CALLBACK( on_toolbar_label_changed ));
 
 		tooltip_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionTooltipEntry" );
-		g_signal_connect(
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( tooltip_widget ),
 				"changed",
-				G_CALLBACK( on_tooltip_changed ),
-				instance );
+				G_CALLBACK( on_tooltip_changed ));
 
 		icon_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionIconComboBoxEntry" );
-		g_signal_connect(
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( GTK_BIN( icon_widget )->child ),
 				"changed",
-				G_CALLBACK( on_icon_changed ),
-				instance );
+				G_CALLBACK( on_icon_changed ));
 
 		button = base_window_get_widget( BASE_WINDOW( instance ), "ActionIconBrowseButton" );
-		g_signal_connect(
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( button ),
 				"clicked",
-				G_CALLBACK( on_icon_browse ),
-				instance );
+				G_CALLBACK( on_icon_browse ));
 
 		enabled_button = get_enabled_button( instance );
-		g_signal_connect(
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
 				G_OBJECT( enabled_button ),
 				"toggled",
-				G_CALLBACK( on_enabled_toggled ),
-				instance );
+				G_CALLBACK( on_enabled_toggled ));
+
+		button = base_window_get_widget( BASE_WINDOW( instance ), "ActionReadonlyButton" );
+		base_window_signal_connect(
+				BASE_WINDOW( instance ),
+				G_OBJECT( button ),
+				"toggled",
+				G_CALLBACK( on_readonly_toggled ));
 	}
 }
 
@@ -302,6 +352,8 @@ nact_iaction_tab_dispose( NactIActionTab *instance )
 	g_return_if_fail( NACT_IS_IACTION_TAB( instance ));
 
 	if( st_initialized && !st_finalized ){
+
+		release_icon_combobox( instance );
 	}
 }
 
@@ -354,14 +406,19 @@ on_tab_updatable_selection_changed( NactIActionTab *instance, gint count_selecte
 	static const gchar *thisfn = "nact_iaction_tab_on_tab_updatable_selection_changed";
 	NAObjectItem *item;
 	gboolean enable_tab;
-	gboolean target_selection, target_background, target_toolbar;
+	gboolean target_selection, target_location, target_toolbar;
 	gboolean enable_label;
 	gboolean same_label;
 	GtkWidget *label_widget, *tooltip_widget, *icon_widget, *title_widget;
 	gchar *label, *tooltip, *icon;
+	GtkButton *icon_button;
 	GtkButton *enabled_button;
+	GtkToggleButton *readonly_button;
 	gboolean enabled_item;
 	GtkToggleButton *toggle;
+	gboolean editable;
+	GtkNotebook *notebook;
+	GtkWidget *page;
 
 	g_debug( "%s: instance=%p, count_selected=%d", thisfn, ( void * ) instance, count_selected );
 	g_return_if_fail( BASE_IS_WINDOW( instance ));
@@ -369,9 +426,12 @@ on_tab_updatable_selection_changed( NactIActionTab *instance, gint count_selecte
 
 	if( st_initialized && !st_finalized ){
 
+		st_on_selection_change = TRUE;
+
 		g_object_get(
 				G_OBJECT( instance ),
 				TAB_UPDATABLE_PROP_EDITED_ACTION, &item,
+				TAB_UPDATABLE_PROP_EDITABLE, &editable,
 				NULL );
 
 		g_return_if_fail( !item || NA_IS_OBJECT_ITEM( item ));
@@ -380,76 +440,122 @@ on_tab_updatable_selection_changed( NactIActionTab *instance, gint count_selecte
 		nact_main_tab_enable_page( NACT_MAIN_WINDOW( instance ), TAB_ACTION, enable_tab );
 
 		target_selection = ( item && (
-				( NA_IS_OBJECT_ACTION( item ) && na_object_action_is_target_selection( NA_OBJECT_ACTION( item ))) ||
+				( NA_IS_OBJECT_ACTION( item ) && na_object_is_target_selection( item )) ||
 				( NA_IS_OBJECT_MENU( item ))));
 
-		target_background = ( item && (
-				( NA_IS_OBJECT_ACTION( item ) && na_object_action_is_target_background( NA_OBJECT_ACTION( item ))) ||
+		target_location = ( item && (
+				( NA_IS_OBJECT_ACTION( item ) && na_object_is_target_location( item )) ||
 				( NA_IS_OBJECT_MENU( item ))));
 
-		target_toolbar = (item && (
-				( NA_IS_OBJECT_ACTION( item ) && na_object_action_is_target_toolbar( NA_OBJECT_ACTION( item )))));
+		target_toolbar = ( item && (
+				( NA_IS_OBJECT_ACTION( item ) && na_object_is_target_toolbar( NA_OBJECT_ACTION( item )))));
 
 		toggle = GTK_TOGGLE_BUTTON( base_window_get_widget( BASE_WINDOW( instance ), "ActionTargetSelectionButton" ));
 		gtk_toggle_button_set_active( toggle, target_selection );
 		gtk_widget_set_sensitive( GTK_WIDGET( toggle ), item && NA_IS_OBJECT_ACTION( item ));
+		nact_gtk_utils_set_editable( GTK_OBJECT( toggle ), editable );
 
-		toggle = GTK_TOGGLE_BUTTON( base_window_get_widget( BASE_WINDOW( instance ), "ActionTargetBackgroundButton" ));
-		gtk_toggle_button_set_active( toggle, target_background );
+		toggle = GTK_TOGGLE_BUTTON( base_window_get_widget( BASE_WINDOW( instance ), "ActionTargetLocationButton" ));
+		gtk_toggle_button_set_active( toggle, target_location );
 		gtk_widget_set_sensitive( GTK_WIDGET( toggle ), item && NA_IS_OBJECT_ACTION( item ));
+		nact_gtk_utils_set_editable( GTK_OBJECT( toggle ), editable );
 
-		enable_label = ( item && ( NA_IS_OBJECT_MENU( item ) || target_selection || target_background ));
+		enable_label = ( item && ( NA_IS_OBJECT_MENU( item ) || target_selection || target_location ));
 		label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionMenuLabelEntry" );
-		label = enable_label ? na_object_get_label( item ) : g_strdup( "" );
+		label = item ? na_object_get_label( item ) : g_strdup( "" );
 		gtk_entry_set_text( GTK_ENTRY( label_widget ), label );
 		if( item ){
 			check_for_label( instance, GTK_ENTRY( label_widget ), label );
 		}
 		g_free( label );
 		gtk_widget_set_sensitive( label_widget, enable_label );
+		nact_gtk_utils_set_editable( GTK_OBJECT( label_widget ), editable );
 
 		toggle = GTK_TOGGLE_BUTTON( base_window_get_widget( BASE_WINDOW( instance ), "ActionTargetToolbarButton" ));
 		gtk_toggle_button_set_active( toggle, target_toolbar );
 		gtk_widget_set_sensitive( GTK_WIDGET( toggle ), item && NA_IS_OBJECT_ACTION( item ));
+		nact_gtk_utils_set_editable( GTK_OBJECT( toggle ), editable );
 
 		toggle = GTK_TOGGLE_BUTTON( base_window_get_widget( BASE_WINDOW( instance ), "ToolbarSameLabelButton" ));
-		same_label = item && NA_IS_OBJECT_ACTION( item ) ? na_object_action_toolbar_use_same_label( NA_OBJECT_ACTION( item )) : FALSE;
+		same_label = item && NA_IS_OBJECT_ACTION( item ) ? na_object_is_toolbar_same_label( NA_OBJECT_ACTION( item )) : FALSE;
 		gtk_toggle_button_set_active( toggle, same_label );
 		toolbar_same_label_set_sensitive( instance, item );
+		nact_gtk_utils_set_editable( GTK_OBJECT( toggle ), editable );
 
-		label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionIconLabelEntry" );
-		label = item && NA_IS_OBJECT_ACTION( item ) ? na_object_action_toolbar_get_label( NA_OBJECT_ACTION( item )) : g_strdup( "" );
+		label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionToolbarLabelEntry" );
+		label = item && NA_IS_OBJECT_ACTION( item ) ? na_object_get_toolbar_label( item ) : g_strdup( "" );
 		gtk_entry_set_text( GTK_ENTRY( label_widget ), label );
 		g_free( label );
 		toolbar_label_set_sensitive( instance, item );
+		nact_gtk_utils_set_editable( GTK_OBJECT( label_widget ), editable );
 
 		tooltip_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionTooltipEntry" );
 		tooltip = item ? na_object_get_tooltip( item ) : g_strdup( "" );
 		gtk_entry_set_text( GTK_ENTRY( tooltip_widget ), tooltip );
 		g_free( tooltip );
+		gtk_widget_set_sensitive( tooltip_widget, item != NULL );
+		nact_gtk_utils_set_editable( GTK_OBJECT( tooltip_widget ), editable );
 
 		icon_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionIconComboBoxEntry" );
 		icon = item ? na_object_get_icon( item ) : g_strdup( "" );
 		gtk_entry_set_text( GTK_ENTRY( GTK_BIN( icon_widget )->child ), icon );
 		g_free( icon );
+		gtk_widget_set_sensitive( icon_widget, item != NULL );
+		nact_gtk_utils_set_editable( GTK_OBJECT( icon_widget ), editable );
 
-		title_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionPropertiesTitle" );
-		if( item && NA_IS_OBJECT_MENU( item )){
-			gtk_label_set_markup( GTK_LABEL( title_widget ), _( "<b>Menu properties</b>" ));
-		} else {
-			gtk_label_set_markup( GTK_LABEL( title_widget ), _( "<b>Action properties</b>" ));
+		icon_button = GTK_BUTTON( base_window_get_widget( BASE_WINDOW( instance ), "ActionIconBrowseButton" ));
+		gtk_widget_set_sensitive( GTK_WIDGET( icon_button ), item != NULL );
+		nact_gtk_utils_set_editable( GTK_OBJECT( icon_button ), editable );
+
+		if( enable_tab ){
+			notebook = GTK_NOTEBOOK( base_window_get_widget( BASE_WINDOW( instance ), "MainNotebook" ));
+			page = gtk_notebook_get_nth_page( notebook, TAB_ACTION );
+			title_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionPropertiesTitle" );
+			if( item && NA_IS_OBJECT_MENU( item )){
+				gtk_notebook_set_tab_label_text( notebook, page, _( "Menu" ));
+				gtk_label_set_markup( GTK_LABEL( title_widget ), _( "<b>Menu properties</b>" ));
+			} else {
+				gtk_notebook_set_tab_label_text( notebook, page, _( "Action" ));
+				gtk_label_set_markup( GTK_LABEL( title_widget ), _( "<b>Action properties</b>" ));
+			}
 		}
 
 		enabled_button = get_enabled_button( instance );
 		enabled_item = item ? na_object_is_enabled( NA_OBJECT_ITEM( item )) : FALSE;
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( enabled_button ), enabled_item );
+		gtk_widget_set_sensitive( GTK_WIDGET( enabled_button ), item != NULL );
+		nact_gtk_utils_set_editable( GTK_OBJECT( enabled_button ), editable );
+
+		/* read-only toggle only indicates the intrinsic writability status of this item
+		 * _not_ the writability status of the provider
+		 */
+		readonly_button = GTK_TOGGLE_BUTTON( base_window_get_widget( BASE_WINDOW( instance ), "ActionReadonlyButton" ));
+		gtk_toggle_button_set_active( readonly_button, item ? na_object_is_readonly( item ) : FALSE );
+		gtk_widget_set_sensitive( GTK_WIDGET( readonly_button ), item != NULL );
+		nact_gtk_utils_set_editable( GTK_OBJECT( readonly_button ), FALSE );
 
 		label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionItemID" );
 		label = item ? na_object_get_id( item ) : g_strdup( "" );
 		gtk_label_set_text( GTK_LABEL( label_widget ), label );
 		g_free( label );
+		gtk_widget_set_sensitive( label_widget, item != NULL );
 
-		/* TODO: manage read-only flag */
+		display_provider_name( instance, item );
+
+		st_on_selection_change = FALSE;
+	}
+}
+
+static void
+on_tab_updatable_provider_changed( NactIActionTab *instance, NAObjectItem *item )
+{
+	static const gchar *thisfn = "nact_iaction_tab_on_tab_updatable_provider_changed";
+
+	g_debug( "%s: instance=%p, item=%p", thisfn, ( void * ) instance, ( void * ) item );
+
+	if( st_initialized && !st_finalized ){
+
+		display_provider_name( instance, item );
 	}
 }
 
@@ -459,45 +565,75 @@ on_target_selection_toggled( GtkToggleButton *button, NactIActionTab *instance )
 	static const gchar *thisfn = "nact_iaction_tab_on_target_selection_toggled";
 	NAObjectAction *action;
 	gboolean is_target;
+	gboolean editable;
 
-	g_debug( "%s: button=%p, instance=%p", thisfn, ( void * ) button, ( void * ) instance );
+	if( !st_on_selection_change ){
+		g_debug( "%s: button=%p, instance=%p", thisfn, ( void * ) button, ( void * ) instance );
 
-	g_object_get(
-			G_OBJECT( instance ),
-			TAB_UPDATABLE_PROP_EDITED_ACTION, &action,
-			NULL );
+		g_object_get(
+				G_OBJECT( instance ),
+				TAB_UPDATABLE_PROP_EDITED_ACTION, &action,
+				TAB_UPDATABLE_PROP_EDITABLE, &editable,
+				NULL );
 
-	if( action && NA_IS_OBJECT_ACTION( action )){
+		g_debug( "%s: item=%p (%s), editable=%s",
+				thisfn, ( void * ) action, action ? G_OBJECT_TYPE_NAME( action ) : "(null)",
+				editable ? "True":"False" );
 
-		is_target = gtk_toggle_button_get_active( button );
-		na_object_action_set_target_selection( action, is_target );
+		if( action && NA_IS_OBJECT_ACTION( action )){
 
-		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ENABLE_TAB, action );
-		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, action, FALSE );
+			is_target = gtk_toggle_button_get_active( button );
+
+			if( editable ){
+				na_object_set_target_selection( action, is_target );
+				g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ENABLE_TAB, action );
+				g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, action, FALSE );
+
+			} else {
+				g_signal_handlers_block_by_func(( gpointer ) button, on_target_selection_toggled, instance );
+				gtk_toggle_button_set_active( button, !is_target );
+				g_signal_handlers_unblock_by_func(( gpointer ) button, on_target_selection_toggled, instance );
+			}
+		}
 	}
 }
 
 static void
-on_target_background_toggled( GtkToggleButton *button, NactIActionTab *instance )
+on_target_location_toggled( GtkToggleButton *button, NactIActionTab *instance )
 {
-	static const gchar *thisfn = "nact_iaction_tab_on_target_background_toggled";
+	static const gchar *thisfn = "nact_iaction_tab_on_target_location_toggled";
 	NAObjectAction *action;
 	gboolean is_target;
+	gboolean editable;
 
-	g_debug( "%s: button=%p, instance=%p", thisfn, ( void * ) button, ( void * ) instance );
+	if( !st_on_selection_change ){
+		g_debug( "%s: button=%p, instance=%p", thisfn, ( void * ) button, ( void * ) instance );
 
-	g_object_get(
-			G_OBJECT( instance ),
-			TAB_UPDATABLE_PROP_EDITED_ACTION, &action,
-			NULL );
+		g_object_get(
+				G_OBJECT( instance ),
+				TAB_UPDATABLE_PROP_EDITED_ACTION, &action,
+				TAB_UPDATABLE_PROP_EDITABLE, &editable,
+				NULL );
 
-	if( action && NA_IS_OBJECT_ACTION( action )){
+		g_debug( "%s: item=%p (%s), editable=%s",
+				thisfn, ( void * ) action, action ? G_OBJECT_TYPE_NAME( action ) : "(null)",
+				editable ? "True":"False" );
 
-		is_target = gtk_toggle_button_get_active( button );
-		na_object_action_set_target_background( action, is_target );
+		if( action && NA_IS_OBJECT_ACTION( action )){
 
-		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ENABLE_TAB, action );
-		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, action, FALSE );
+			is_target = gtk_toggle_button_get_active( button );
+
+			if( editable ){
+				na_object_set_target_location( action, is_target );
+				g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ENABLE_TAB, action );
+				g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, action, FALSE );
+
+			} else {
+				g_signal_handlers_block_by_func(( gpointer ) button, on_target_location_toggled, instance );
+				gtk_toggle_button_set_active( button, !is_target );
+				g_signal_handlers_unblock_by_func(( gpointer ) button, on_target_location_toggled, instance );
+			}
+		}
 	}
 }
 
@@ -540,8 +676,6 @@ on_label_changed( GtkEntry *entry, NactIActionTab *instance )
 {
 	NAObjectItem *edited;
 	const gchar *label;
-	gboolean target_toolbar;
-	gboolean toolbar_same_label;
 
 	g_object_get(
 			G_OBJECT( instance ),
@@ -554,13 +688,7 @@ on_label_changed( GtkEntry *entry, NactIActionTab *instance )
 		check_for_label( instance, entry, label );
 
 		if( NA_IS_OBJECT_ACTION( edited )){
-			target_toolbar = na_object_action_is_target_toolbar( NA_OBJECT_ACTION( edited ));
-			if( target_toolbar ){
-				toolbar_same_label = na_object_action_toolbar_use_same_label( NA_OBJECT_ACTION( edited ));
-				if( toolbar_same_label ){
-					na_object_action_toolbar_set_label( NA_OBJECT_ACTION( edited ), label );
-				}
-			}
+			setup_toolbar_label( instance, edited, label );
 		}
 
 		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, edited, TRUE );
@@ -568,15 +696,14 @@ on_label_changed( GtkEntry *entry, NactIActionTab *instance )
 }
 
 static void
-set_label_label( NactIActionTab *instance, const gchar *color )
+set_label_label( NactIActionTab *instance, const gchar *color_str )
 {
 	GtkWidget *label;
-	gchar *text;
+	GdkColor color;
 
 	label = base_window_get_widget( BASE_WINDOW( instance ), "ActionMenuLabelLabel" );
-	/* i18n: label in front of the GtkEntry where user enters the action label */
-	text = g_markup_printf_escaped( "<span color=\"%s\">%s</span>", color, _( "_Menu label :" ));
-	gtk_label_set_markup_with_mnemonic( GTK_LABEL( label ), text );
+	gdk_color_parse( color_str, &color );
+	gtk_widget_modify_fg( label, GTK_STATE_NORMAL, &color );
 }
 
 static void
@@ -585,48 +712,79 @@ on_target_toolbar_toggled( GtkToggleButton *button, NactIActionTab *instance )
 	static const gchar *thisfn = "nact_iaction_tab_on_target_toolbar_toggled";
 	NAObjectAction *action;
 	gboolean is_target;
+	gboolean editable;
 
-	g_debug( "%s: button=%p, instance=%p", thisfn, ( void * ) button, ( void * ) instance );
+	if( !st_on_selection_change ){
+		g_debug( "%s: button=%p, instance=%p", thisfn, ( void * ) button, ( void * ) instance );
 
-	g_object_get(
-			G_OBJECT( instance ),
-			TAB_UPDATABLE_PROP_EDITED_ACTION, &action,
-			NULL );
+		g_object_get(
+				G_OBJECT( instance ),
+				TAB_UPDATABLE_PROP_EDITED_ACTION, &action,
+				TAB_UPDATABLE_PROP_EDITABLE, &editable,
+				NULL );
 
-	if( action && NA_IS_OBJECT_ACTION( action )){
+		if( action && NA_IS_OBJECT_ACTION( action )){
 
-		is_target = gtk_toggle_button_get_active( button );
-		na_object_action_set_target_toolbar( action, is_target );
+			is_target = gtk_toggle_button_get_active( button );
 
-		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ENABLE_TAB, action );
-		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, action, FALSE );
+			if( editable ){
+				na_object_set_target_toolbar( action, is_target );
+				g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ENABLE_TAB, action );
+				g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, action, FALSE );
+				toolbar_same_label_set_sensitive( instance, NA_OBJECT_ITEM( action ));
+				toolbar_label_set_sensitive( instance, NA_OBJECT_ITEM( action ));
+
+			} else {
+				g_signal_handlers_block_by_func(( gpointer ) button, on_target_toolbar_toggled, instance );
+				gtk_toggle_button_set_active( button, !is_target );
+				g_signal_handlers_unblock_by_func(( gpointer ) button, on_target_toolbar_toggled, instance );
+			}
+		}
 	}
-
-	toolbar_same_label_set_sensitive( instance, NA_OBJECT_ITEM( action ));
-	toolbar_label_set_sensitive( instance, NA_OBJECT_ITEM( action ));
 }
 
 static void
 on_toolbar_same_label_toggled( GtkToggleButton *button, NactIActionTab *instance )
 {
+	static const gchar *thisfn = "nact_iaction_tab_on_toolbar_same_label_toggled";
 	NAObjectItem *edited;
 	gboolean same_label;
+	gboolean editable;
+	gchar *label;
+	GtkWidget *label_widget;
 
-	g_object_get(
-			G_OBJECT( instance ),
-			TAB_UPDATABLE_PROP_EDITED_ACTION, &edited,
-			NULL );
+	if( !st_on_selection_change ){
+		g_debug( "%s: button=%p, instance=%p", thisfn, ( void * ) button, ( void * ) instance );
 
-	if( edited && NA_IS_OBJECT_ACTION( edited )){
+		g_object_get(
+				G_OBJECT( instance ),
+				TAB_UPDATABLE_PROP_EDITED_ACTION, &edited,
+				TAB_UPDATABLE_PROP_EDITABLE, &editable,
+				NULL );
 
-		same_label = gtk_toggle_button_get_active( button );
-		na_object_action_toolbar_set_same_label( NA_OBJECT_ACTION( edited ), same_label );
+		if( edited && NA_IS_OBJECT_ACTION( edited )){
 
-		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, edited, FALSE );
+			same_label = gtk_toggle_button_get_active( button );
+
+			if( editable ){
+				na_object_set_toolbar_same_label( NA_OBJECT_ACTION( edited ), same_label );
+				if( same_label ){
+					label = na_object_get_label( edited );
+					label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionToolbarLabelEntry" );
+					gtk_entry_set_text( GTK_ENTRY( label_widget ), label );
+					g_free( label );
+				}
+				g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, edited, FALSE );
+				toolbar_same_label_set_sensitive( instance, NA_OBJECT_ITEM( edited ));
+				toolbar_label_set_sensitive( instance, NA_OBJECT_ITEM( edited ));
+
+			} else {
+				g_signal_handlers_block_by_func(( gpointer ) button, on_toolbar_same_label_toggled, instance );
+				gtk_toggle_button_set_active( button, !same_label );
+				g_signal_handlers_unblock_by_func(( gpointer ) button, on_toolbar_same_label_toggled, instance );
+			}
+		}
 	}
-
-	toolbar_same_label_set_sensitive( instance, NA_OBJECT_ITEM( edited ));
-	toolbar_label_set_sensitive( instance, NA_OBJECT_ITEM( edited ));
 }
 
 static void
@@ -634,10 +792,28 @@ toolbar_same_label_set_sensitive( NactIActionTab *instance, NAObjectItem *item )
 {
 	GtkToggleButton *toggle;
 	gboolean target_toolbar;
+	gboolean readonly;
 
+	readonly = item ? na_object_is_readonly( item ) : FALSE;
 	toggle = GTK_TOGGLE_BUTTON( base_window_get_widget( BASE_WINDOW( instance ), "ToolbarSameLabelButton" ));
-	target_toolbar = item && NA_IS_OBJECT_ACTION( item ) ? na_object_action_is_target_toolbar( NA_OBJECT_ACTION( item )) : FALSE;
-	gtk_widget_set_sensitive( GTK_WIDGET( toggle ), target_toolbar );
+	target_toolbar = item && NA_IS_OBJECT_ACTION( item ) ? na_object_is_target_toolbar( NA_OBJECT_ACTION( item )) : FALSE;
+	gtk_widget_set_sensitive( GTK_WIDGET( toggle ), target_toolbar && !readonly );
+}
+
+/*
+ * setup the label of the toolbar according to the toolbar_same_label flag
+ */
+static void
+setup_toolbar_label( NactIActionTab *instance, NAObjectItem *item, const gchar *label )
+{
+	GtkWidget *label_widget;
+
+	if( item && NA_IS_OBJECT_ACTION( item )){
+		if( na_object_is_toolbar_same_label( item )){
+			label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionToolbarLabelEntry" );
+			gtk_entry_set_text( GTK_ENTRY( label_widget ), label );
+		}
+	}
 }
 
 static void
@@ -651,11 +827,10 @@ on_toolbar_label_changed( GtkEntry *entry, NactIActionTab *instance )
 			TAB_UPDATABLE_PROP_EDITED_ACTION, &edited,
 			NULL );
 
-	if( edited ){
-		g_return_if_fail( NA_IS_OBJECT_ACTION( edited ));
+	if( edited && NA_IS_OBJECT_ACTION( edited )){
 
 		label = gtk_entry_get_text( entry );
-		na_object_action_toolbar_set_label( NA_OBJECT_ACTION( edited ), label );
+		na_object_set_toolbar_label( NA_OBJECT_ACTION( edited ), label );
 
 		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, edited, FALSE );
 	}
@@ -669,8 +844,8 @@ toolbar_label_set_sensitive( NactIActionTab *instance, NAObjectItem *item )
 	GtkWidget *label_widget;
 
 	is_action = item && NA_IS_OBJECT_ACTION( item );
-	same_label = is_action ? na_object_action_toolbar_use_same_label( NA_OBJECT_ACTION( item )) : FALSE;
-	label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionIconLabelEntry" );
+	same_label = is_action ? na_object_is_toolbar_same_label( NA_OBJECT_ACTION( item )) : FALSE;
+	label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionToolbarLabelEntry" );
 	gtk_widget_set_sensitive( label_widget, is_action && !same_label );
 }
 
@@ -685,7 +860,7 @@ on_tooltip_changed( GtkEntry *entry, NactIActionTab *instance )
 			NULL );
 
 	if( edited ){
-		na_object_item_set_tooltip( edited, gtk_entry_get_text( entry ));
+		na_object_set_tooltip( edited, gtk_entry_get_text( entry ));
 		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, edited, FALSE );
 	}
 }
@@ -713,7 +888,7 @@ create_stock_icon_model( void )
 	stock_list = g_slist_sort( stock_list, ( GCompareFunc ) sort_stock_ids );
 
 	for( iter = stock_list ; iter ; iter = iter->next ){
-		icon_info = gtk_icon_theme_lookup_icon( icon_theme, ( gchar * ) iter->data, GTK_ICON_SIZE_MENU, GTK_ICON_LOOKUP_FORCE_SVG );
+		icon_info = gtk_icon_theme_lookup_icon( icon_theme, ( gchar * ) iter->data, GTK_ICON_SIZE_MENU, GTK_ICON_LOOKUP_GENERIC_FALLBACK );
 		if( icon_info ){
 			if( gtk_stock_lookup(( gchar * ) iter->data, &stock_item )){
 				gtk_list_store_append( model, &row );
@@ -731,21 +906,6 @@ create_stock_icon_model( void )
 	return( GTK_TREE_MODEL( model ));
 }
 
-static void
-display_icon( NactIActionTab *instance, GtkWidget *image, gboolean show )
-{
-	GtkFrame *frame = GTK_FRAME( base_window_get_widget( BASE_WINDOW( instance ), "ActionIconFrame" ));
-
-	if( show ){
-		gtk_widget_show( image );
-		gtk_frame_set_shadow_type( frame, GTK_SHADOW_NONE );
-
-	} else {
-		gtk_widget_hide( image );
-		gtk_frame_set_shadow_type( frame, GTK_SHADOW_IN );
-	}
-}
-
 static void
 icon_combo_list_fill( GtkComboBoxEntry* combo )
 {
@@ -800,15 +960,15 @@ on_icon_browse( GtkButton *button, NactIActionTab *instance )
 static void
 on_icon_changed( GtkEntry *icon_entry, NactIActionTab *instance )
 {
-	/*static const gchar *thisfn = "nact_iaction_tab_on_icon_changed";*/
-	GtkWidget *image;
-	GdkPixbuf *icon;
+	static const gchar *thisfn = "nact_iaction_tab_on_icon_changed";
+	GtkImage *image;
+	GdkPixbuf *pixbuf;
 	NAObjectItem *edited;
 	const gchar *icon_name;
 
-	image = base_window_get_widget( BASE_WINDOW( instance ), "ActionIconImage" );
-	g_assert( GTK_IS_WIDGET( image ));
-	display_icon( instance, image, FALSE );
+	g_debug( "%s: entry=%p, instance=%p", thisfn, ( void * ) icon_entry, ( void * ) instance );
+
+	icon_name = NULL;
 
 	g_object_get(
 			G_OBJECT( instance ),
@@ -817,14 +977,16 @@ on_icon_changed( GtkEntry *icon_entry, NactIActionTab *instance )
 
 	if( edited ){
 		icon_name = gtk_entry_get_text( icon_entry );
-		na_object_item_set_icon( edited, icon_name );
+		na_object_set_icon( edited, icon_name );
 		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, edited, TRUE );
+	}
 
-		if( icon_name && strlen( icon_name ) > 0 ){
-			icon = na_object_item_get_pixbuf( edited, image );
-			gtk_image_set_from_pixbuf( GTK_IMAGE( image ), icon );
-			display_icon( instance, image, TRUE );
-		}
+	pixbuf = nact_gtk_utils_get_pixbuf( icon_name, GTK_ICON_SIZE_MENU );
+
+	if( pixbuf ){
+		image = GTK_IMAGE( base_window_get_widget( BASE_WINDOW( instance ), "ActionIconImage" ));
+		g_assert( GTK_IS_WIDGET( image ));
+		gtk_image_set_from_pixbuf( image, pixbuf );
 	}
 }
 
@@ -880,6 +1042,17 @@ strip_underscore( const gchar *text )
 	return( result );
 }
 
+static void
+release_icon_combobox( NactIActionTab *instance )
+{
+	GtkWidget *combo;
+	GtkTreeModel *model;
+
+	combo = base_window_get_widget( BASE_WINDOW( instance ), "ActionIconComboBoxEntry" );
+	model = gtk_combo_box_get_model( GTK_COMBO_BOX( combo ));
+	gtk_list_store_clear( GTK_LIST_STORE( model ));
+}
+
 static GtkButton *
 get_enabled_button( NactIActionTab *instance )
 {
@@ -889,17 +1062,73 @@ get_enabled_button( NactIActionTab *instance )
 static void
 on_enabled_toggled( GtkToggleButton *button, NactIActionTab *instance )
 {
+	static const gchar *thisfn = "nact_iaction_tab_on_enabled_toggled";
 	NAObjectItem *edited;
 	gboolean enabled;
+	gboolean editable;
 
-	g_object_get(
-			G_OBJECT( instance ),
-			TAB_UPDATABLE_PROP_EDITED_ACTION, &edited,
-			NULL );
+	if( !st_on_selection_change ){
+		g_debug( "%s: button=%p, instance=%p", thisfn, ( void * ) button, ( void * ) instance );
 
-	if( edited && NA_IS_OBJECT_ITEM( edited )){
-		enabled = gtk_toggle_button_get_active( button );
-		na_object_item_set_enabled( edited, enabled );
-		g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, edited, FALSE );
+		g_object_get(
+				G_OBJECT( instance ),
+				TAB_UPDATABLE_PROP_EDITED_ACTION, &edited,
+				TAB_UPDATABLE_PROP_EDITABLE, &editable,
+				NULL );
+
+		if( edited && NA_IS_OBJECT_ITEM( edited )){
+
+			enabled = gtk_toggle_button_get_active( button );
+
+			if( editable ){
+				na_object_set_enabled( edited, enabled );
+				g_signal_emit_by_name( G_OBJECT( instance ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, edited, FALSE );
+
+			} else {
+				g_signal_handlers_block_by_func(( gpointer ) button, on_enabled_toggled, instance );
+				gtk_toggle_button_set_active( button, !enabled );
+				g_signal_handlers_unblock_by_func(( gpointer ) button, on_enabled_toggled, instance );
+			}
+		}
 	}
 }
+
+static void
+on_readonly_toggled( GtkToggleButton *button, NactIActionTab *instance )
+{
+	static const gchar *thisfn = "nact_iaction_tab_on_readonly_toggled";
+	gboolean active;
+
+	if( !st_on_selection_change ){
+		g_debug( "%s: button=%p, instance=%p", thisfn, ( void * ) button, ( void * ) instance );
+
+		active = gtk_toggle_button_get_active( button );
+
+		g_signal_handlers_block_by_func(( gpointer ) button, on_readonly_toggled, instance );
+		gtk_toggle_button_set_active( button, !active );
+		g_signal_handlers_unblock_by_func(( gpointer ) button, on_readonly_toggled, instance );
+	}
+}
+
+static void
+display_provider_name( NactIActionTab *instance, NAObjectItem *item )
+{
+	GtkWidget *label_widget;
+	gchar *label;
+	NAIOProvider *provider;
+
+	label_widget = base_window_get_widget( BASE_WINDOW( instance ), "ActionItemProvider" );
+	label = NULL;
+	if( item ){
+		provider = na_object_get_provider( item );
+		if( provider ){
+			label = na_io_provider_get_name( provider );
+		}
+	}
+	if( !label ){
+		label = g_strdup( "" );
+	}
+	gtk_label_set_text( GTK_LABEL( label_widget ), label );
+	g_free( label );
+	gtk_widget_set_sensitive( label_widget, item != NULL );
+}
