@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,25 +37,28 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include <runtime/na-pivot.h>
-#include <runtime/na-iabout.h>
-#include <runtime/na-iio-provider.h>
-#include <runtime/na-ipivot-consumer.h>
+#include <api/na-object-api.h>
 
-#include <common/na-object-api.h>
-#include <common/na-iprefs.h>
+#include <core/na-iabout.h>
+#include <core/na-ipivot-consumer.h>
+#include <core/na-iprefs.h>
+#include <core/na-pivot.h>
 
 #include "base-iprefs.h"
 #include "nact-application.h"
 #include "nact-iactions-list.h"
 #include "nact-iaction-tab.h"
 #include "nact-icommand-tab.h"
-#include "nact-ibackground-tab.h"
+#include "nact-ifolders-tab.h"
 #include "nact-iconditions-tab.h"
 #include "nact-iadvanced-tab.h"
 #include "nact-main-tab.h"
 #include "nact-main-menubar.h"
+#include "nact-main-statusbar.h"
+#include "nact-marshal.h"
 #include "nact-main-window.h"
+#include "nact-confirm-logout.h"
+#include "nact-sort-buttons.h"
 
 /* private class data
  */
@@ -78,8 +81,13 @@ struct NactMainWindowPrivate {
 	 * it may be different of the row being currently selected.
 	 *
 	 * Can be null, and this implies that edited_profile is also null.
+	 *
+	 * 'editable_item' property is computed on selection change;
+	 * This is the real writability status of the item.
 	 */
 	NAObjectItem    *edited_item;
+	gboolean         editable;
+	gint             reason;
 
 	/**
 	 * Currently edited profile.
@@ -107,16 +115,20 @@ struct NactMainWindowPrivate {
 };
 
 /* action properties
+ * these are set when selection changes as an optimization try
  */
 enum {
 	PROP_EDITED_ITEM = 1,
 	PROP_EDITED_PROFILE,
-	PROP_SELECTED_ROW
+	PROP_SELECTED_ROW,
+	PROP_EDITABLE,
+	PROP_REASON
 };
 
 /* signals
  */
 enum {
+	PROVIDER_CHANGED,
 	SELECTION_CHANGED,
 	ITEM_UPDATED,
 	ENABLE_TAB,
@@ -133,7 +145,7 @@ static void     class_init( NactMainWindowClass *klass );
 static void     iactions_list_iface_init( NactIActionsListInterface *iface );
 static void     iaction_tab_iface_init( NactIActionTabInterface *iface );
 static void     icommand_tab_iface_init( NactICommandTabInterface *iface );
-static void     ibackground_tab_iface_init( NactIBackgroundTabInterface *iface );
+static void     ifolders_tab_iface_init( NactIFoldersTabInterface *iface );
 static void     iconditions_tab_iface_init( NactIConditionsTabInterface *iface );
 static void     iadvanced_tab_iface_init( NactIAdvancedTabInterface *iface );
 static void     iabout_iface_init( NAIAboutInterface *iface );
@@ -145,14 +157,16 @@ static void     instance_set_property( GObject *object, guint property_id, const
 static void     instance_dispose( GObject *application );
 static void     instance_finalize( GObject *application );
 
-static void     actually_delete_item( NactMainWindow *window, NAObject *item, NAPivot *pivot );
+static void     actually_delete_item( NactMainWindow *window, NAObject *item, NAUpdater *updater );
 
-static gchar   *base_get_toplevel_name( BaseWindow *window );
-static gchar   *base_get_iprefs_window_id( BaseWindow *window );
+static gchar   *base_get_toplevel_name( const BaseWindow *window );
+static gchar   *base_get_iprefs_window_id( const BaseWindow *window );
+static gboolean base_is_willing_to_quit( const BaseWindow *window );
 static void     on_base_initial_load_toplevel( NactMainWindow *window, gpointer user_data );
 static void     on_base_runtime_init_toplevel( NactMainWindow *window, gpointer user_data );
 static void     on_base_all_widgets_showed( NactMainWindow *window, gpointer user_data );
 
+static void     on_main_window_level_zero_order_changed( NactMainWindow *window, gpointer user_data );
 static void     on_iactions_list_selection_changed( NactIActionsList *instance, GSList *selected_items );
 static void     on_iactions_list_status_changed( NactMainWindow *window, gpointer user_data );
 static void     set_current_object_item( NactMainWindow *window, GSList *selected_items );
@@ -164,8 +178,9 @@ static void     on_tab_updatable_item_updated( NactMainWindow *window, gpointer
 
 static gboolean confirm_for_giveup_from_menu( NactMainWindow *window );
 static gboolean confirm_for_giveup_from_pivot( NactMainWindow *window );
-static void     ipivot_consumer_on_actions_changed( NAIPivotConsumer *instance, gpointer user_data );
+static void     ipivot_consumer_on_items_changed( NAIPivotConsumer *instance, gpointer user_data );
 static void     ipivot_consumer_on_display_order_changed( NAIPivotConsumer *instance, gint order_mode );
+static void     ipivot_consumer_on_mandatory_prefs_changed( NAIPivotConsumer *instance );
 static void     reload( NactMainWindow *window );
 
 static gchar   *iabout_get_application_name( NAIAbout *instance );
@@ -218,8 +233,8 @@ register_type( void )
 		NULL
 	};
 
-	static const GInterfaceInfo ibackground_tab_iface_info = {
-		( GInterfaceInitFunc ) ibackground_tab_iface_init,
+	static const GInterfaceInfo ifolders_tab_iface_info = {
+		( GInterfaceInitFunc ) ifolders_tab_iface_init,
 		NULL,
 		NULL
 	};
@@ -264,7 +279,7 @@ register_type( void )
 
 	g_type_add_interface_static( type, NACT_ICOMMAND_TAB_TYPE, &icommand_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_IBACKGROUND_TAB_TYPE, &ibackground_tab_iface_info );
+	g_type_add_interface_static( type, NACT_IFOLDERS_TAB_TYPE, &ifolders_tab_iface_info );
 
 	g_type_add_interface_static( type, NACT_ICONDITIONS_TAB_TYPE, &iconditions_tab_iface_info );
 
@@ -318,19 +333,55 @@ class_init( NactMainWindowClass *klass )
 			G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE );
 	g_object_class_install_property( object_class, PROP_SELECTED_ROW, spec );
 
+	spec = g_param_spec_boolean(
+			TAB_UPDATABLE_PROP_EDITABLE,
+			"Editable item ?",
+			"Whether the item will be able to be updated against its I/O provider", FALSE,
+			G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE );
+	g_object_class_install_property( object_class, PROP_EDITABLE, spec );
+
+	spec = g_param_spec_int(
+			TAB_UPDATABLE_PROP_REASON,
+			"No edition reason",
+			"Why is this item not editable", 0, 255, 0,
+			G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE );
+	g_object_class_install_property( object_class, PROP_REASON, spec );
+
 	klass->private = g_new0( NactMainWindowClassPrivate, 1 );
 
 	base_class = BASE_WINDOW_CLASS( klass );
 	base_class->get_toplevel_name = base_get_toplevel_name;
 	base_class->get_iprefs_window_id = base_get_iprefs_window_id;
+	base_class->is_willing_to_quit = base_is_willing_to_quit;
 
 	/**
-	 * nact-tab-updatable-selection-changed:
+	 * nact-tab-updatable-provider-changed:
+	 *
+	 * This signal is emitted at save time, when we are noticing that
+	 * the save operation has led to a modification of the I/O provider.
+	 * This signal may be caught by a tab in order to display the
+	 * new provider's name.
+	 */
+	st_signals[ PROVIDER_CHANGED ] = g_signal_new(
+			TAB_UPDATABLE_SIGNAL_PROVIDER_CHANGED,
+			G_TYPE_OBJECT,
+			G_SIGNAL_RUN_LAST,
+			0,					/* no default handler */
+			NULL,
+			NULL,
+			g_cclosure_marshal_VOID__POINTER,
+			G_TYPE_NONE,
+			1,
+			G_TYPE_POINTER );
+
+	/**
+	 * main-window-selection-changed:
 	 *
 	 * This signal is emitted by this main window, in response of a
-	 * change of the selection in IActionsList.
+	 * change of the selection in IActionsList, after having updated
+	 * its properties.
 	 * Notebook tabs should connect to this signal and update their
-	 * display to reflect the content of the new selection.
+	 * display to reflect the content of the new selection (if applyable).
 	 *
 	 * Note also that, where this main window will receive from
 	 * IActionsList the full list of currently selected items, this
@@ -339,7 +390,7 @@ class_init( NactMainWindowClass *klass )
 	 * See #iactions_list_selection_changed().
 	 */
 	st_signals[ SELECTION_CHANGED ] = g_signal_new(
-			TAB_UPDATABLE_SIGNAL_SELECTION_CHANGED,
+			MAIN_WINDOW_SIGNAL_SELECTION_CHANGED,
 			G_TYPE_OBJECT,
 			G_SIGNAL_RUN_LAST,
 			0,					/* no default handler */
@@ -367,10 +418,11 @@ class_init( NactMainWindowClass *klass )
 			0,					/* no default handler */
 			NULL,
 			NULL,
-			g_cclosure_marshal_VOID__POINTER,
+			nact_marshal_VOID__POINTER_BOOLEAN,
 			G_TYPE_NONE,
-			1,
-			G_TYPE_POINTER );
+			2,
+			G_TYPE_POINTER,
+			G_TYPE_BOOLEAN );
 
 	/**
 	 * nact-tab-updatable-enable-tab:
@@ -435,7 +487,6 @@ iactions_list_iface_init( NactIActionsListInterface *iface )
 
 	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 
-	iface->selection_changed = NULL;
 	iface->get_treeview_name = iactions_list_get_treeview_name;
 }
 
@@ -456,9 +507,9 @@ icommand_tab_iface_init( NactICommandTabInterface *iface )
 }
 
 static void
-ibackground_tab_iface_init( NactIBackgroundTabInterface *iface )
+ifolders_tab_iface_init( NactIFoldersTabInterface *iface )
 {
-	static const gchar *thisfn = "nact_main_window_ibackground_tab_iface_init";
+	static const gchar *thisfn = "nact_main_window_ifolders_tab_iface_init";
 
 	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 }
@@ -496,10 +547,11 @@ ipivot_consumer_iface_init( NAIPivotConsumerInterface *iface )
 
 	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
 
-	iface->on_actions_changed = ipivot_consumer_on_actions_changed;
+	iface->on_items_changed = ipivot_consumer_on_items_changed;
 	iface->on_create_root_menu_changed = NULL;
 	iface->on_display_about_changed = NULL;
 	iface->on_display_order_changed = ipivot_consumer_on_display_order_changed;
+	iface->on_mandatory_prefs_changed = ipivot_consumer_on_mandatory_prefs_changed;
 }
 
 static void
@@ -516,7 +568,8 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	static const gchar *thisfn = "nact_main_window_instance_init";
 	NactMainWindow *self;
 
-	g_debug( "%s: instance=%p, klass=%p", thisfn, ( void * ) instance, ( void * ) klass );
+	g_debug( "%s: instance=%p (%s), klass=%p",
+			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( instance ));
 	self = NACT_MAIN_WINDOW( instance );
 
@@ -568,10 +621,18 @@ instance_get_property( GObject *object, guint property_id, GValue *value, GParam
 				g_value_set_pointer( value, self->private->edited_profile );
 				break;
 
-			case PROP_SELECTED_ROW	:
+			case PROP_SELECTED_ROW:
 				g_value_set_pointer( value, self->private->selected_row );
 				break;
 
+			case PROP_EDITABLE:
+				g_value_set_boolean( value, self->private->editable );
+				break;
+
+			case PROP_REASON:
+				g_value_set_int( value, self->private->reason );
+				break;
+
 			default:
 				G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, spec );
 				break;
@@ -602,6 +663,14 @@ instance_set_property( GObject *object, guint property_id, const GValue *value,
 				self->private->selected_row = g_value_get_pointer( value );
 				break;
 
+			case PROP_EDITABLE:
+				self->private->editable = g_value_get_boolean( value );
+				break;
+
+			case PROP_REASON:
+				self->private->reason = g_value_get_int( value );
+				break;
+
 			default:
 				G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, spec );
 				break;
@@ -618,7 +687,7 @@ instance_dispose( GObject *window )
 	gint pos;
 	GList *it;
 
-	g_debug( "%s: window=%p", thisfn, ( void * ) window );
+	g_debug( "%s: window=%p (%s)", thisfn, ( void * ) window, G_OBJECT_TYPE_NAME( window ));
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( window ));
 	self = NACT_MAIN_WINDOW( window );
 
@@ -633,14 +702,15 @@ instance_dispose( GObject *window )
 		base_iprefs_set_int( BASE_WINDOW( window ), "main-paned", pos );
 
 		for( it = self->private->deleted ; it ; it = it->next ){
-			g_debug( "nact_main_window_instance_dispose: %p (%s)", ( void * ) it->data, G_OBJECT_TYPE_NAME( it->data ));
+			g_debug( "nact_main_window_instance_dispose: deleted=%p (%s)", ( void * ) it->data, G_OBJECT_TYPE_NAME( it->data ));
 		}
-		na_object_free_items_list( self->private->deleted );
+		na_object_unref_items( self->private->deleted );
 
 		nact_iactions_list_dispose( NACT_IACTIONS_LIST( window ));
+		nact_sort_buttons_dispose( self );
 		nact_iaction_tab_dispose( NACT_IACTION_TAB( window ));
 		nact_icommand_tab_dispose( NACT_ICOMMAND_TAB( window ));
-		nact_ibackground_tab_dispose( NACT_IBACKGROUND_TAB( window ));
+		nact_ifolders_tab_dispose( NACT_IFOLDERS_TAB( window ));
 		nact_iconditions_tab_dispose( NACT_ICONDITIONS_TAB( window ));
 		nact_iadvanced_tab_dispose( NACT_IADVANCED_TAB( window ));
 		nact_main_menubar_dispose( self );
@@ -719,7 +789,7 @@ nact_main_window_get_item( const NactMainWindow *window, const gchar *uuid )
 {
 	NAObjectItem *exists;
 	NactApplication *application;
-	NAPivot *pivot;
+	NAUpdater *updater;
 
 	g_return_val_if_fail( NACT_IS_MAIN_WINDOW( window ), FALSE );
 
@@ -730,12 +800,12 @@ nact_main_window_get_item( const NactMainWindow *window, const gchar *uuid )
 		/* leave here this dead code, in case I change of opinion later */
 		if( 0 ){
 			application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-			pivot = nact_application_get_pivot( application );
-			exists = NA_OBJECT_ITEM( na_pivot_get_item( pivot, uuid ));
+			updater = nact_application_get_updater( application );
+			exists = na_pivot_get_item( NA_PIVOT( updater ), uuid );
 		}
 
 		if( !exists ){
-			exists = NA_OBJECT_ITEM( nact_iactions_list_get_item( NACT_IACTIONS_LIST( window ), uuid ));
+			exists = NA_OBJECT_ITEM( nact_iactions_list_bis_get_item( NACT_IACTIONS_LIST( window ), uuid ));
 		}
 	}
 
@@ -755,8 +825,6 @@ nact_main_window_get_item( const NactMainWindow *window, const gchar *uuid )
  *    we can create any new actions, deleting them, and so on
  *    if we have eventually deleted all newly created actions, then the
  *    final count of modified actions should be zero... don't it ?
- *
- * Note also that we don't count invalid items as they are not saveable.
  */
 gboolean
 nact_main_window_has_modified_items( const NactMainWindow *window )
@@ -782,7 +850,7 @@ nact_main_window_has_modified_items( const NactMainWindow *window )
 		g_debug( "%s: has_modified=%s", thisfn, has_modified ? "True":"False" );
 	}
 
-	return( count_deleted > 0 || has_modified );
+	return( count_deleted > 0 || has_modified || nact_main_menubar_is_level_zero_order_changed( window ));
 }
 
 /**
@@ -853,7 +921,7 @@ void
 nact_main_window_remove_deleted( NactMainWindow *window )
 {
 	NactApplication *application;
-	NAPivot *pivot;
+	NAUpdater *updater;
 	GList *it;
 	NAObject *item;
 
@@ -862,15 +930,17 @@ nact_main_window_remove_deleted( NactMainWindow *window )
 	if( !window->private->dispose_has_run ){
 
 		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-		pivot = nact_application_get_pivot( application );
+		updater = nact_application_get_updater( application );
 
 		for( it = window->private->deleted ; it ; it = it->next ){
 			item = NA_OBJECT( it->data );
-			actually_delete_item( window, item, pivot );
+			actually_delete_item( window, item, updater );
 		}
 
-		na_object_free_items_list( window->private->deleted );
+		na_object_unref_items( window->private->deleted );
 		window->private->deleted = NULL;
+
+		setup_dialog_title( window );
 	}
 }
 
@@ -880,7 +950,7 @@ nact_main_window_remove_deleted( NactMainWindow *window )
  * and thus updated in the storage subsystem as well as in the pivot
  */
 static void
-actually_delete_item( NactMainWindow *window, NAObject *item, NAPivot *pivot )
+actually_delete_item( NactMainWindow *window, NAObject *item, NAUpdater *updater )
 {
 	GList *items, *it;
 	NAObject *origin;
@@ -892,31 +962,47 @@ actually_delete_item( NactMainWindow *window, NAObject *item, NAPivot *pivot )
 		nact_window_delete_item( NACT_WINDOW( window ), NA_OBJECT_ITEM( item ));
 
 		if( NA_IS_OBJECT_MENU( item )){
-			items = na_object_get_items_list( item );
+			items = na_object_get_items( item );
 			for( it = items ; it ; it = it->next ){
-				actually_delete_item( window, NA_OBJECT( it->data ), pivot );
+				actually_delete_item( window, NA_OBJECT( it->data ), updater );
 			}
 		}
 
-		origin = na_object_get_origin( item );
+		origin = ( NAObject * ) na_object_get_origin( item );
 		if( origin ){
-			na_pivot_remove_item( pivot, origin );
+			na_updater_remove_item( updater, origin );
 		}
 	}
 }
 
 static gchar *
-base_get_toplevel_name( BaseWindow *window )
+base_get_toplevel_name( const BaseWindow *window )
 {
 	return( g_strdup( "MainWindow" ));
 }
 
 static gchar *
-base_get_iprefs_window_id( BaseWindow *window )
+base_get_iprefs_window_id( const BaseWindow *window )
 {
 	return( g_strdup( "main-window" ));
 }
 
+static gboolean
+base_is_willing_to_quit( const BaseWindow *window )
+{
+	static const gchar *thisfn = "nact_main_window_is_willing_to_quit";
+	gboolean willing_to;
+
+	g_debug( "%s: window=%p", thisfn, ( void * ) window );
+
+	willing_to = TRUE;
+	if( nact_main_window_has_modified_items( NACT_MAIN_WINDOW( window ))){
+		willing_to = nact_confirm_logout_run( NACT_MAIN_WINDOW( window ));
+	}
+
+	return( willing_to );
+}
+
 /*
  * note that for this NactMainWindow, on_initial_load_toplevel and
  * on_runtime_init_toplevel are equivalent, as there is only one
@@ -943,12 +1029,15 @@ on_base_initial_load_toplevel( NactMainWindow *window, gpointer user_data )
 
 		nact_iactions_list_set_management_mode( NACT_IACTIONS_LIST( window ), IACTIONS_LIST_MANAGEMENT_MODE_EDITION );
 		nact_iactions_list_initial_load_toplevel( NACT_IACTIONS_LIST( window ));
+		nact_sort_buttons_initial_load( window );
 
 		nact_iaction_tab_initial_load_toplevel( NACT_IACTION_TAB( window ));
 		nact_icommand_tab_initial_load_toplevel( NACT_ICOMMAND_TAB( window ));
-		nact_ibackground_tab_initial_load_toplevel( NACT_IBACKGROUND_TAB( window ));
+		nact_ifolders_tab_initial_load_toplevel( NACT_IFOLDERS_TAB( window ));
 		nact_iconditions_tab_initial_load_toplevel( NACT_ICONDITIONS_TAB( window ));
 		nact_iadvanced_tab_initial_load_toplevel( NACT_IADVANCED_TAB( window ));
+
+		nact_main_statusbar_initial_load_toplevel( window );
 	}
 }
 
@@ -957,7 +1046,7 @@ on_base_runtime_init_toplevel( NactMainWindow *window, gpointer user_data )
 {
 	static const gchar *thisfn = "nact_main_window_on_base_runtime_init_toplevel";
 	NactApplication *application;
-	NAPivot *pivot;
+	NAUpdater *updater;
 	GList *tree;
 	gint order_mode;
 
@@ -975,23 +1064,24 @@ on_base_runtime_init_toplevel( NactMainWindow *window, gpointer user_data )
 				G_CALLBACK( on_iactions_list_selection_changed ));
 
 		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-		pivot = nact_application_get_pivot( application );
-		tree = na_pivot_get_items( pivot );
+		updater = nact_application_get_updater( application );
+		tree = na_pivot_get_items( NA_PIVOT( updater ));
 		g_debug( "%s: pivot_tree=%p", thisfn, ( void * ) tree );
 
 		nact_iaction_tab_runtime_init_toplevel( NACT_IACTION_TAB( window ));
 		nact_icommand_tab_runtime_init_toplevel( NACT_ICOMMAND_TAB( window ));
-		nact_ibackground_tab_runtime_init_toplevel( NACT_IBACKGROUND_TAB( window ));
+		nact_ifolders_tab_runtime_init_toplevel( NACT_IFOLDERS_TAB( window ));
 		nact_iconditions_tab_runtime_init_toplevel( NACT_ICONDITIONS_TAB( window ));
 		nact_iadvanced_tab_runtime_init_toplevel( NACT_IADVANCED_TAB( window ));
 		nact_main_menubar_runtime_init( window );
 
-		order_mode = na_iprefs_get_order_mode( NA_IPREFS( pivot ));
+		order_mode = na_iprefs_get_order_mode( NA_IPREFS( updater ));
 		ipivot_consumer_on_display_order_changed( NA_IPIVOT_CONSUMER( window ), order_mode );
 
 		/* fill the IActionsList at last so that all signals are connected
 		 */
 		nact_iactions_list_runtime_init_toplevel( NACT_IACTIONS_LIST( window ), tree );
+		nact_sort_buttons_runtime_init( window );
 
 		/* this to update the title when an item is modified
 		 */
@@ -1000,6 +1090,12 @@ on_base_runtime_init_toplevel( NactMainWindow *window, gpointer user_data )
 				G_OBJECT( window ),
 				IACTIONS_LIST_SIGNAL_STATUS_CHANGED,
 				G_CALLBACK( on_iactions_list_status_changed ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( window ),
+				G_OBJECT( window ),
+				MAIN_WINDOW_SIGNAL_LEVEL_ZERO_ORDER_CHANGED,
+				G_CALLBACK( on_main_window_level_zero_order_changed ));
 	}
 }
 
@@ -1013,21 +1109,31 @@ on_base_all_widgets_showed( NactMainWindow *window, gpointer user_data )
 	g_return_if_fail( NACT_IS_IACTIONS_LIST( window ));
 	g_return_if_fail( NACT_IS_IACTION_TAB( window ));
 	g_return_if_fail( NACT_IS_ICOMMAND_TAB( window ));
-	g_return_if_fail( NACT_IS_IBACKGROUND_TAB( window ));
+	g_return_if_fail( NACT_IS_IFOLDERS_TAB( window ));
 	g_return_if_fail( NACT_IS_ICONDITIONS_TAB( window ));
 	g_return_if_fail( NACT_IS_IADVANCED_TAB( window ));
 
 	if( !window->private->dispose_has_run ){
 
-		nact_iactions_list_all_widgets_showed( NACT_IACTIONS_LIST( window ));
 		nact_iaction_tab_all_widgets_showed( NACT_IACTION_TAB( window ));
 		nact_icommand_tab_all_widgets_showed( NACT_ICOMMAND_TAB( window ));
-		nact_ibackground_tab_all_widgets_showed( NACT_IBACKGROUND_TAB( window ));
+		nact_ifolders_tab_all_widgets_showed( NACT_IFOLDERS_TAB( window ));
 		nact_iconditions_tab_all_widgets_showed( NACT_ICONDITIONS_TAB( window ));
 		nact_iadvanced_tab_all_widgets_showed( NACT_IADVANCED_TAB( window ));
+
+		nact_iactions_list_all_widgets_showed( NACT_IACTIONS_LIST( window ));
+		nact_sort_buttons_all_widgets_showed( window );
 	}
 }
 
+static void
+on_main_window_level_zero_order_changed( NactMainWindow *window, gpointer user_data )
+{
+	g_debug( "nact_main_window_on_main_window_level_zero_order_changed" );
+
+	setup_dialog_title( window );
+}
+
 /*
  * iactions_list_selection_changed:
  * @window: this #NactMainWindow instance.
@@ -1040,6 +1146,8 @@ on_iactions_list_selection_changed( NactIActionsList *instance, GSList *selected
 	NactMainWindow *window;
 	NAObject *object;
 	gint count;
+	NactApplication *application;
+	NAUpdater *updater;
 
 	count = g_slist_length( selected_items );
 
@@ -1052,6 +1160,12 @@ on_iactions_list_selection_changed( NactIActionsList *instance, GSList *selected
 		return;
 	}
 
+	window->private->selected_row = NULL;
+	window->private->edited_item = NULL;
+	window->private->editable = FALSE;
+	window->private->reason = 0;
+	nact_main_statusbar_set_locked( window, FALSE, 0 );
+
 	if( count == 1 ){
 		g_return_if_fail( NA_IS_OBJECT_ID( selected_items->data ));
 		object = NA_OBJECT( selected_items->data );
@@ -1067,26 +1181,25 @@ on_iactions_list_selection_changed( NactIActionsList *instance, GSList *selected
 			set_current_profile( window, TRUE, selected_items );
 		}
 
+		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( instance )));
+		updater = nact_application_get_updater( application );
+		window->private->editable = na_updater_is_item_writable( updater, window->private->edited_item, &window->private->reason );
+		nact_main_statusbar_set_locked( window, !window->private->editable, window->private->reason );
+
 	} else {
-		window->private->selected_row = NULL;
-		window->private->edited_item = NULL;
 		set_current_object_item( window, selected_items );
 	}
 
-	g_object_set(
-			G_OBJECT( window ),
-			TAB_UPDATABLE_PROP_EDITED_ACTION, window->private->edited_item,
-			TAB_UPDATABLE_PROP_EDITED_PROFILE, window->private->edited_profile,
-			NULL );
-
 	setup_dialog_title( window );
 
-	g_signal_emit_by_name( window, TAB_UPDATABLE_SIGNAL_SELECTION_CHANGED, GINT_TO_POINTER( count ));
+	g_signal_emit_by_name( window, MAIN_WINDOW_SIGNAL_SELECTION_CHANGED, GINT_TO_POINTER( count ));
 }
 
 static void
 on_iactions_list_status_changed( NactMainWindow *window, gpointer user_data )
 {
+	g_debug( "nact_main_window_on_iactions_list_status_changed" );
+
 	setup_dialog_title( window );
 }
 
@@ -1110,16 +1223,18 @@ set_current_object_item( NactMainWindow *window, GSList *selected_items )
 	 */
 	window->private->edited_profile = NULL;
 
-	if( window->private->edited_item &&
-		NA_IS_OBJECT_ACTION( window->private->edited_item )){
+	if( window->private->edited_item ){
 
-			count_profiles = na_object_get_items_count( NA_OBJECT_ACTION( window->private->edited_item ));
+		if( NA_IS_OBJECT_ACTION( window->private->edited_item )){
+
+			count_profiles = na_object_get_items_count( window->private->edited_item );
 			/*g_return_if_fail( count_profiles >= 1 );*/
 
 			if( count_profiles == 1 ){
-				profiles = na_object_get_items_list( window->private->edited_item );
+				profiles = na_object_get_items( window->private->edited_item );
 				window->private->edited_profile = NA_OBJECT_PROFILE( profiles->data );
 			}
+		}
 	}
 
 	set_current_profile( window, FALSE, selected_items );
@@ -1134,8 +1249,12 @@ set_current_profile( NactMainWindow *window, gboolean set_action, GSList *select
 			thisfn, ( void * ) window, set_action ? "True":"False", ( void * ) selected_items );
 
 	if( window->private->edited_profile && set_action ){
+
 		NAObjectAction *action = NA_OBJECT_ACTION( na_object_get_parent( window->private->edited_profile ));
+		NactApplication *application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
+		NAUpdater *updater = nact_application_get_updater( application );
 		window->private->edited_item = NA_OBJECT_ITEM( action );
+		window->private->editable = na_updater_is_item_writable( updater, window->private->edited_item, &window->private->reason );
 	}
 }
 
@@ -1151,6 +1270,12 @@ iactions_list_get_treeview_name( NactIActionsList *instance )
 	return( name );
 }
 
+/*
+ * the title bar of the main window brings up three informations:
+ * - the name of the application
+ * - the name of the currently selected item if there is only one
+ * - an asterisk if anything has been modified
+ */
 static void
 setup_dialog_title( NactMainWindow *window )
 {
@@ -1160,6 +1285,7 @@ setup_dialog_title( NactMainWindow *window )
 	gchar *label;
 	gchar *tmp;
 
+	g_debug( "setup_dialog_title" );
 	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
 	title = base_application_get_application_name( BASE_APPLICATION( application ));
 
@@ -1266,9 +1392,9 @@ confirm_for_giveup_from_pivot( NactMainWindow *window )
  * reset - saving is handled on a per-action basis.
  */
 static void
-ipivot_consumer_on_actions_changed( NAIPivotConsumer *instance, gpointer user_data )
+ipivot_consumer_on_items_changed( NAIPivotConsumer *instance, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_main_window_ipivot_consumer_on_actions_changed";
+	static const gchar *thisfn = "nact_main_window_ipivot_consumer_on_items_changed";
 	NactMainWindow *window;
 	gboolean reload_ok;
 
@@ -1289,20 +1415,27 @@ ipivot_consumer_on_actions_changed( NAIPivotConsumer *instance, gpointer user_da
 static void
 reload( NactMainWindow *window )
 {
+	static const gchar *thisfn = "nact_main_window_reload";
 	NactApplication *application;
-	NAPivot *pivot;
+	NAUpdater *updater;
 
+	g_debug( "%s: window=%p", thisfn, ( void * ) window );
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( window ));
 
 	if( !window->private->dispose_has_run ){
 
-		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-		pivot = nact_application_get_pivot( application );
-		na_pivot_reload_items( pivot );
-		nact_iactions_list_fill( NACT_IACTIONS_LIST( window ), na_pivot_get_items( pivot ));
+		window->private->edited_item = NULL;
+		window->private->edited_profile = NULL;
+		window->private->selected_row = NULL;
 
-		na_object_free_items_list( window->private->deleted );
+		na_object_unref_items( window->private->deleted );
 		window->private->deleted = NULL;
+
+		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
+		updater = nact_application_get_updater( application );
+		na_pivot_load_items( NA_PIVOT( updater ));
+		nact_iactions_list_fill( NACT_IACTIONS_LIST( window ), na_pivot_get_items( NA_PIVOT( updater )));
+		nact_iactions_list_bis_select_first_row( NACT_IACTIONS_LIST( window ));
 	}
 }
 
@@ -1321,6 +1454,16 @@ ipivot_consumer_on_display_order_changed( NAIPivotConsumer *instance, gint order
 	/*self = NACT_MAIN_WINDOW( instance );*/
 
 	nact_iactions_list_display_order_change( NACT_IACTIONS_LIST( instance ), order_mode );
+	nact_sort_buttons_display_order_change( NACT_MAIN_WINDOW( instance ), order_mode );
+
+	g_signal_emit_by_name(
+			NACT_MAIN_WINDOW( instance ), MAIN_WINDOW_SIGNAL_LEVEL_ZERO_ORDER_CHANGED, GINT_TO_POINTER( TRUE ));
+}
+
+static void
+ipivot_consumer_on_mandatory_prefs_changed( NAIPivotConsumer *instance )
+{
+	nact_sort_buttons_level_zero_writability_change( NACT_MAIN_WINDOW( instance ));
 }
 
 static gchar *
